#ifndef DATABASE_H
#define DATABASE_H

#include <QtCore>
#include "networkmonitor.h"

typedef BitBudget::NetworkMonitor::data __nwm_d;

namespace BitBudget {
namespace DataBase {


typedef struct {
    QDateTime start;
    QDateTime finish;
} date_range;

typedef struct {
    ulong monthly_rx;
    ulong monthly_tx;
    ulong daily_rx;
    ulong daily_tx;
    ulong hourly_rx;
    ulong hourly_tx;
} overview_data;


/*!
    \brief Returns a list of file names in the database.
*/
QList<QString> list();

/*!
 * \return A list of DateTimes representing hours logged in the database.
*/
QList<QDateTime> list_dates();

/*!
    \brief Returns the date_range for the recorded data.
*/
date_range range();

/*!
    \brief Returns a list of QDateTimes for every hour in a date_range including the start and finish.
*/
QList<QDateTime> hours (date_range d);

/*!
    \brief Returns a list of QDateTimes for every day in a date_range including the start and finish.
*/
QList<QDateTime> days (date_range d);

/*!
    \brief Returns a list of QDateTimes for every month in a date_range including the start and finish.
*/
QList<QDateTime> months (date_range d, int rotate);

/*!
    \brief Returns a list of QDateTimes for every month in a date_range including the start and finish.
*/
QList<QDateTime> months (date_range d);

/*!
    \brief Returns a date_range for a specified month, starting and ending on day n.
    \param year An integer representing the year - e.g. 2012.
    \param month An integer specifying the month (January=1, December=12).
    \param n An integer between 1 and 28 inclusive representing the start and end day of the month.
*/
date_range month (int year, int month, int n);

/*!
    \brief Returns a date_range for a specified month.
    \param year An integer representing the year - e.g. 2012.
    \param month An integer specifying the month (January=1, December=12).
*/
date_range month (int year, int month);

/*!
    \brief Returns a date_range for a specified day.
*/
date_range day (int year, int month, int day);

/*!
    \brief Gets the path of the database file for a date-time to the nearest hour.
*/
QByteArray path(QDateTime t);
/*!
    \brief Gets the path of the database file for a year, month, day and hour.
*/
QByteArray path(int year, int month, int day, int hour);
/*!
    \brief Reads a database file for a date-time.
*/
__nwm_d read(QDateTime t);
/*!
    \brief Reads a database file for a year, month, day and hour.
*/
__nwm_d read(int year, int month, int day, int hour);
/*!
    \brief Writes a per interface data usage structure to a file for a date-time.
*/
void write(__nwm_d d, QDateTime t);
/*!
    \brief Writes a per interface data usage structure to a file for a year, month, day and hour.
*/
void write(__nwm_d d, int year, int month, int day, int hour);

/*!
 * \brief Gets the total data usage on each interface from the database for a given date range.
*/
__nwm_d get_usage(date_range range);

/*!
 * \brief Gets the total data usage for a given month.
 * \param year The year part of the date.
 * \param month The month part of the date.
 * \param rotate The day that the month starts and ends on (1-28).
*/
__nwm_d get_monthly_usage(int rotate, int year, int month);
/*!
 * \brief Gets the total data usage this month (with the 1st as the start date).
*/
__nwm_d get_monthly_usage();
/*!
 * \brief Gets the total data usage for this month.
 * \param rotate The day that the month starts and ends on (1-28).
*/
__nwm_d get_monthly_usage(int rotate);
/*!
 * \brief Gets the total data usage for a given month.
 * \param month The nth month before today. 0=this month, 1=last month etc.
 * \param rotate The day that the month starts and ends on (1-28).
*/
__nwm_d get_monthly_usage(int rotate, int month);

/*!
 * \brief Gets the total data usage for a given day.
 * \param year The year part of the date.
 * \param month The month part of the date.
 * \param day The day part of the date.
*/
__nwm_d get_daily_usage(int year, int month, int day);
/*!
 * \brief Gets the total data usage for today.
*/
__nwm_d get_daily_usage();
/*!
 * \brief Gets the total data usage for a given day.
 * \param day The nth day before today. 0=today, 1=yesterday etc.
*/
__nwm_d get_daily_usage(int day);

/*!
 * \brief Gets the total data usage for a given hour.
 * \param year The year part of the date.
 * \param month The month part of the date.
 * \param day The day part of the date.
 * \param hour The hour.
*/
__nwm_d get_hourly_usage(int year, int month, int day, int hour);
/*!
 * \brief Gets the total data usage today.
*/
__nwm_d get_hourly_usage();
/*!
 * \brief Gets the total data usage for a given hour.
 * \param hour The nth hour before this hour. 0=this hour, 1=last hour etc.
*/
__nwm_d get_hourly_usage(int hour);

/*!
 * \brief Gets an overview of the total data use on the named interfaces.
 * \param month_rotate The day of the month that months start and end on. (1-28)
 * \param ifaces The interfaces to take data from.
*/
overview_data get_overview(int month_rotate, QList<QByteArray> ifaces);
/*!
 * \brief Gets an overview of the total data use on the named interfaces.
 * \param ifaces The interfaces to take data from.
*/
overview_data get_overview(QList<QByteArray> ifaces);
/*!
 * \brief Gets an overview of the total data use on all interfaces (except lo).
*/
overview_data get_overview();

}
}


#endif // DATABASE_H
