#ifndef NETWORKMONITOR_H
#define NETWORKMONITOR_H


#include <QtCore>

#include "common.h"

namespace BitBudget {
    namespace NetworkMonitor {

        /*!
            \brief Data type representing an amount of data usage.
            rx is the number of bytes received (downloaded).
            tx is the number of bytes transmitted (uploaded).
        */
        typedef struct {
            ulong rx;
            ulong tx;
        } data_t;

        /*!
            \brief Data type representing the data usage of network interfaces.
            The keys of the hash are the names of network interfaces (e.g. "eth0").
        */
        typedef QHash<QByteArray, data_t> data;


        /*!
            \brief Reads a file in the format of /proc/net/dev, returning the per-interface data usage.
            \param file A file-like object, implementing the interface QIODevice.
        */
        data get_data (QObject &file);
        /*!
            \brief Reads a file in the format of /proc/net/dev, returning the per-interface data usage.
            \param filename The path to the file on the filesystem.
        */
        data get_data (QString filename);
        /*!
            \brief Reads /proc/net/dev, returining the per-interface data usage since boot.
        */
        data get_data ();

        /*!
            \brief Returns a per-interface data usage structure, with all values as 0.
        */
        data data_zero ();

        /*!
            \brief Calculates the difference between 2 sets of data usages.
        */
        data data_delta (data a, data b);

        /*!
            \brief Calculates the sum of 2 sets of data usages.
        */
        data data_sum (data a, data b);

        /*!
            \brief Calculates the difference between the current data usage (from /proc/net/dev) and initial.
        */
        data data_delta (data initial);

        /*!
            \brief Calculates a total data usage for all the named interfaces for a data usage structure.
            \param d The data usage structure.
            \param ifaces A list of network interfaces to aggregate.
        */
        data_t data_aggregate (data d, QList<QByteArray> ifaces);

        /*!
            \brief Calculates a total data usage for all the named interfaces, using data from /proc/net/dev.
            \param ifaces A list of network interfaces to aggregate.
        */
        data_t data_aggregate (QList<QByteArray> ifaces);

        /*!
            \brief Loads a data usage data structure from a file.
            \param file File object - object implementing QIODevice.
        */
        data load_data (QObject &file);

        /*!
            \brief Loads a data usage data structure from a file.
            \param filename Path to file on filesystem.
        */
        data load_data (QByteArray filename);

        /*!
            \brief Saves a data usage data structure to a file.
            \param file File object - writable object implementing QIODevice.
            \param d The data structure to be written.
        */
        void save_data (QObject &file, data &d);

        /*!
            \brief Saves a data usage data structure to a file.
            \param filename File Path to file on filesystem.
            \param d The data structure to be written.
        */
        void save_data (QByteArray filename, data &d);

        /*!
            \brief Returns a list of all network interfaces on the system.
        */
        QList<QByteArray> ifaces_all ();
        /*!
            \brief Returns a list of network interfaces on the system, excluding lo.
        */
        QList<QByteArray> ifaces ();
        /*!
            \brief Returns a list of network interfaces on the system, excluding those named.
        */
        QList<QByteArray> ifaces (QList<QByteArray> exclude);

    }
}

#endif // NETWORKMONITOR_H
