#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "database.h"
#include "networkmonitor.h"
#include "common.h"
#include "config.h"
#include "aboutdialog.h"
#include <unistd.h>

#include <QCheckBox>
#include <QGraphicsTextItem>



MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    this->showMinimized();
    this->setWindowFlags(Qt::Window | Qt::WindowTitleHint | Qt::CustomizeWindowHint | Qt::WindowMinimizeButtonHint);

    this->has_initialised = false;
    this->_d = true;
    this->config.load();
    bool config_binary = this->config.binary;
    ulong config_download_quota = this->config.download_quota;
    ulong config_upload_quota = this->config.upload_quota;
    ulong config_total_quota = this->config.total_quota;
    int config_month_rotate = this->config.month_rotate;
    QList<QByteArray> config_ifaces = this->config.ifaces;

    QDateTime now = QDateTime::currentDateTime();
    ui->setupUi(this);

    quicklist = new QMenu(this);
    quicklist_exporter = new DBusMenuExporter("/com/lazyteq/bitbudget/quicklist", quicklist);
    signal_properties["quicklist"] = "/com/lazyteq/bitbudget/quicklist";
    signal_properties["progress-visible"] = true;

    if (config_binary) {
        ui->base2radio->setChecked(true);
    } else {
        ui->base10radio->setChecked(true);
    }

    ui->downquotaunit->setCurrentIndex(this->get_unit_index(config_download_quota));
    ui->upquotaunit->setCurrentIndex(this->get_unit_index(config_upload_quota));
    ui->totalquotaunit->setCurrentIndex(this->get_unit_index(config_total_quota));
    ulong unit = this->get_unit(ui->downquotaunit);
    ui->downquota->setValue(config_download_quota / unit);
    unit = this->get_unit(ui->upquotaunit);
    ui->upquota->setValue(config_upload_quota / unit);
    unit = this->get_unit(ui->totalquotaunit);
    ui->totalquota->setValue(config_total_quota / unit);

    ui->monthrotate->setValue(config_month_rotate);

    QList<QByteArray> ifaces = BitBudget::NetworkMonitor::ifaces_all();
    QListIterator<QByteArray> i(ifaces);
    QByteArray iface;
    QCheckBox *cb;
    QString iface_text;
    while (i.hasNext()) {
        cb = new QCheckBox(this);
        QObject::connect(cb, SIGNAL(clicked()), this, SLOT(onsettingschanged()));
        iface = i.next();
        if (iface == "lo") {
            iface_text = "Loopback Interface (lo)";
        } else {
            if (iface.startsWith("wlan")) {
                iface_text = "Wireless Network (";
                iface_text.append(iface);
                iface_text.append(")");
            } else {
                if (iface.startsWith("eth")) {
                    iface_text = "Wired Ethernet Network (";
                    iface_text.append(iface);
                    iface_text.append(")");
                } else {
                    iface_text = iface;
                }
            }
        }
        cb->setText(iface_text);
        cb->setChecked(config_ifaces.contains(iface));
        ui->ifacevbox->addWidget(cb);
    }

    this->has_initialised = true;

    ui->dateTimeEdit->setDateTime(now.addMonths(-3));
    ui->dateTimeEdit_2->setDateTime(now);
    ui->dateTimeEdit_3->setDateTime(now.addDays(-1));
    ui->dateTimeEdit_4->setDateTime(now);

    this->scene = new QGraphicsScene(this);
    ui->chart->setScene(this->scene);
    ui->chart->setAlignment(Qt::AlignLeft | Qt::AlignTop);
    ui->chart->setDragMode(QGraphicsView::ScrollHandDrag);

    timer = new QTimer(this);
    timer->setInterval(5000);
    QObject::connect(timer, SIGNAL(timeout()), this, SLOT(timerTimeout()));
    timer->start();

    this->refresh();
}

void MainWindow::go_preferences () {
    this->showNormal();
    ui->tabWidget->setCurrentIndex(2);
}

void MainWindow::go_hourly (bool do_set) {
    BitBudget::DataBase::date_range dr = BitBudget::DataBase::range();
    ui->dateTimeEdit_3->setDateTime(dr.finish.addSecs(-24*HOUR));
    ui->dateTimeEdit_4->setDateTime(dr.finish);
    if (do_set) {
        ui->comboBox->setCurrentIndex(0);
        ui->tabWidget->setCurrentIndex(1);
    }
}

void MainWindow::quicklist_hourly_activate() {
    this->unminimise();
    go_hourly(true);
}

void MainWindow::go_daily (bool do_set) {
    BitBudget::DataBase::date_range dr = BitBudget::DataBase::range();
    ui->dateTimeEdit_3->setDateTime(dr.finish.addDays(-30));
    ui->dateTimeEdit_4->setDateTime(dr.finish);
    if (do_set) {
        ui->comboBox->setCurrentIndex(1);
        ui->tabWidget->setCurrentIndex(1);
    }
}

void MainWindow::quicklist_daily_activate() {
    this->unminimise();
    go_daily(true);
}

void MainWindow::go_monthly (bool do_set) {
    BitBudget::DataBase::date_range dr = BitBudget::DataBase::range();
    dr = BitBudget::DataBase::range();
    ui->dateTimeEdit_3->setDateTime(dr.finish.addMonths(-12));
    ui->dateTimeEdit_4->setDateTime(dr.finish);
    if (do_set) {
        ui->comboBox->setCurrentIndex(2);
        ui->tabWidget->setCurrentIndex(1);
    }
}

void MainWindow::quicklist_monthly_activate() {
    this->unminimise();
    go_monthly(true);
}

void MainWindow::unminimise() {
    this->activateWindow();
    this->showNormal();
}

void MainWindow::process_arguments (QStringList args) {
    if (args.contains("--show")) {
        this->unminimise();
    }
    if (args.contains("--preferences")) {
        go_preferences();
    } else {
        if (args.contains("--hourly")) {
            go_hourly(true);
        } else {
            if (args.contains("--daily")) {
                go_daily(true);
            } else {
                if (args.contains("--monthly")) {
                    go_monthly(true);
                }
            }
        }
    }
}

int MainWindow::get_unit_index (ulong value) {
    if (value >= GiB)
        return 6;
    if (value >= GB)
        return 5;
    if (value >= MiB)
        return 4;
    if (value >= MB)
        return 3;
    if (value >= KiB)
        return 2;
    if (value >= KB)
        return 1;
    return 0;
}

void MainWindow::resizeEvent (QResizeEvent *event) {
    this->draw_chart();
}

void MainWindow::closeEvent (QCloseEvent *event) {

}

void MainWindow::timerTimeout() {
    this->refresh();
}

void MainWindow::refresh() {
    //Group Box Titles

    QList<QString> sl = ui->gb_h->title().split(" ");
    QString s;
    QTextStream t(&s);
    t << sl[0] << " " << QTime::currentTime().toString("hh:00");
    ui->gb_h->setTitle(s);

    sl = ui->gb_d->title().split(" - ");
    s.clear();
    t << sl[0] << " - " << QDate::currentDate().toString("dd MMM yyyy");
    ui->gb_d->setTitle(s);

    sl = ui->gb_m->title().split(" - ");
    s.clear();
    t << sl[0] << " - " << QDate::currentDate().toString("MMM yyyy");
    ui->gb_m->setTitle(s);

    //Data Usage Overview
    BitBudget::DataBase::overview_data o = BitBudget::DataBase::get_overview(
                this->config.month_rotate,
                this->config.ifaces);

    ui->l_h_d->setText(this->config.format_bytes(o.hourly_rx));
    ui->l_h_u->setText(this->config.format_bytes(o.hourly_tx));
    ui->l_h_t->setText(this->config.format_bytes(o.hourly_rx + o.hourly_tx));

    ui->l_d_d->setText(this->config.format_bytes(o.daily_rx));
    ui->l_d_u->setText(this->config.format_bytes(o.daily_tx));
    ui->l_d_t->setText(this->config.format_bytes(o.daily_rx + o.daily_tx));

    ui->l_m_d->setText(this->config.format_bytes(o.monthly_rx));
    ui->l_m_u->setText(this->config.format_bytes(o.monthly_tx));
    ui->l_m_t->setText(this->config.format_bytes(o.monthly_rx + o.monthly_tx));

    //Historic Data Usage
    BitBudget::DataBase::date_range d;
    d.start = ui->dateTimeEdit->dateTime();
    d.finish = ui->dateTimeEdit_2->dateTime();

    BitBudget::NetworkMonitor::data_t u = BitBudget::NetworkMonitor::data_aggregate(
                BitBudget::DataBase::get_usage(d), this->config.ifaces);

    ui->l_a_d->setText(this->config.format_bytes(u.rx));
    ui->l_a_u->setText(this->config.format_bytes(u.tx));
    ui->l_a_t->setText(this->config.format_bytes(u.rx + u.tx));

    update_launcher(o);

}

void MainWindow::draw_chart() {if (this->_d){

    this->scene->clear();

    QColor down_col(239,41,41);
    QColor up_col(138,226,52);
    QColor tot_col(114,159,207);
    QBrush down_brush(down_col);
    QBrush up_brush(up_col);
    QBrush tot_brush(tot_col);
    QBrush black_brush(QColor(0,0,0));
    QPen pen(QColor(0,0,0,0));

    BitBudget::DataBase::date_range dr;
    dr.start = ui->dateTimeEdit_3->dateTime();
    dr.finish = ui->dateTimeEdit_4->dateTime();
    QList<QDateTime> h;
    QString fmt;
    int marg;
    switch (ui->comboBox->currentIndex()) {
    case 1: //daily
        h = BitBudget::DataBase::days(dr);
        fmt = "dd MMM yyyy";
        marg = 100;
        break;
    case 2: //monthly
        h = BitBudget::DataBase::months(dr, this->config.month_rotate);
        fmt = "MMM yyyy";
        marg = 70;
        break;
    default: //hourly
        h = BitBudget::DataBase::hours(dr);
        fmt = "hh:00  dd MMM yyyy";
        marg = 150;
        break;
    }

    qSort(h.begin(), h.end(), qGreater<QDateTime>());
    QListIterator<QDateTime> i(h);
    QDateTime dt;
    QList<BitBudget::NetworkMonitor::data_t> bins;
    QList<QString> labels;
    long double maxval = 0;
    ulong tt;
    BitBudget::NetworkMonitor::data_t val;
    long double m = 1;

    switch (ui->comboBox->currentIndex()) {
    case 1: //daily
        m = 0.032;
        while (i.hasNext()) {
            dt = i.next();
            val = BitBudget::NetworkMonitor::data_aggregate(
                            BitBudget::DataBase::get_daily_usage(
                                dt.date().year(), dt.date().month(), dt.date().day()),
                            this->config.ifaces);
            tt = val.rx + val.tx;
            if (tt > maxval) {
                maxval = tt;
            }
            bins.append(val);
            labels.append(dt.toString(fmt));
        }
        break;
    case 2: //monthly
        while (i.hasNext()) {
            dt = i.next();
            val = BitBudget::NetworkMonitor::data_aggregate(
                            BitBudget::DataBase::get_monthly_usage(this->config.month_rotate,
                                dt.date().year(), dt.date().month()),
                            this->config.ifaces);
            tt = val.rx + val.tx;
            if (tt > maxval) {
                maxval = tt;
            }
            bins.append(val);
            labels.append(dt.toString(fmt));
        }
        break;
    default: //hourly
        m = 0.0013;
        while (i.hasNext()) {
            dt = i.next();
            val = BitBudget::NetworkMonitor::data_aggregate(
                            BitBudget::DataBase::get_hourly_usage(
                                dt.date().year(), dt.date().month(), dt.date().day(), dt.time().hour()),
                            this->config.ifaces);
            tt = val.rx + val.tx;
            if (tt > maxval) {
                maxval = tt;
            }
            bins.append(val);
            labels.append(dt.toString(fmt));
        }
        break;
    }

    if (this->config.download_quota*m > maxval) {
        maxval = this->config.download_quota*m;
    }
    if (this->config.upload_quota*m > maxval) {
        maxval = this->config.upload_quota*m;
    }
    if (this->config.total_quota*m > maxval) {
        maxval = this->config.total_quota*m;
    }

    if (maxval == 0) {
        maxval = 1;
    }

    int x = 0;
    int y = 0;
    int ind = 0;
    long double factor = 0.0;

    while (ind < bins.length()) {
        this->scene->addText(labels[ind])->setPos(x, y);
        factor = bins[ind].rx / maxval;
        y += 20;
        this->scene->addRect(x + marg, y, factor*(ui->chart->width() - 20 - marg), 20, pen, down_brush);
        this->scene->addText(this->config.format_bytes(bins[ind].rx))->setPos(x + marg, y - 3);
        y += 20;
        factor = bins[ind].tx / maxval;
        this->scene->addRect(x + marg, y, factor*(ui->chart->width() - 20 - marg), 20, pen, up_brush);
        this->scene->addText(this->config.format_bytes(bins[ind].tx))->setPos(x + marg, y - 3);
        y += 20;
        factor = (bins[ind].rx + bins[ind].tx) / maxval;
        this->scene->addRect(x + marg, y, factor*(ui->chart->width() - 20 - marg), 20, pen, tot_brush);
        this->scene->addText(this->config.format_bytes(
                                 bins[ind].rx + bins[ind].tx))->setPos(x + marg, y - 3);
        y += 40;
        ind += 1;
    }

    this->scene->addRect(x + marg - 2, 0, 2, y,
                         pen, black_brush);

    if (this->config.download_quota > 0) {
        factor = (m*this->config.download_quota) / maxval;
        this->scene->addRect(x + marg + factor*(ui->chart->width() - 20 - marg), 0, 2, y,
                             pen, down_brush);
    }
    if (this->config.upload_quota > 0) {
        factor = (m*this->config.upload_quota) / maxval;
        this->scene->addRect(x + marg + factor*(ui->chart->width() - 20 - marg), 0, 2, y,
                             pen, up_brush);
    }
    if (this->config.total_quota > 0) {
        factor = (m*this->config.total_quota) / maxval;
        this->scene->addRect(x + marg + factor*(ui->chart->width() - 20 - marg), 0, 2, y,
                             pen, tot_brush);
    }

    ui->chart->centerOn(0, 0);

}}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_dateTimeEdit_dateTimeChanged(const QDateTime &date)
{
    this->refresh();
}

void MainWindow::on_dateTimeEdit_2_dateTimeChanged(const QDateTime &date)
{
    this->refresh();
}



void MainWindow::on_comboBox_currentIndexChanged(int index)
{
    switch (index) {
    case 0:
        go_hourly(false);
        break;
    case 1:
        go_daily(false);
        break;
    case 2:
        go_monthly(false);
        break;
    }

    this->draw_chart();
}

void MainWindow::on_dateTimeEdit_3_editingFinished()
{
    this->draw_chart();
}

void MainWindow::on_dateTimeEdit_4_editingFinished()
{
    this->draw_chart();
}

ulong MainWindow::get_unit(QComboBox *w) {
    switch (w->currentIndex()) {
    case 1:
        return KB;
    case 2:
        return KiB;
    case 3:
        return MB;
    case 4:
        return MiB;
    case 5:
        return GB;
    case 6:
        return GiB;
    default:
        return 1;
    }
}

void MainWindow::onsettingschanged () {
    if (this->has_initialised) {
        this->config.binary = ui->base2radio->isChecked();
        this->config.month_rotate = ui->monthrotate->value();
        this->config.download_quota = ui->downquota->value() * this->get_unit(ui->downquotaunit);
        this->config.upload_quota = ui->upquota->value() * this->get_unit(ui->upquotaunit);
        this->config.total_quota = ui->totalquota->value() * this->get_unit(ui->totalquotaunit);
        this->config.indicator = false; //deprecated
        this->config.notifications = false; //deprecated
        this->config.ifaces.clear();
        int i = 0;
        QWidget *cb;
        QList<QByteArray> ifcs = BitBudget::NetworkMonitor::ifaces_all();
        while (i < ui->ifacevbox->count() && i < ifcs.length()) {
            cb = ui->ifacevbox->itemAt(i)->widget();
            if (cb->property("checked").toBool()) {
                this->config.ifaces.append(ifcs[i]);
            }
            i++;
        }

        this->config.save();
    }
}

void MainWindow::on_downquota_editingFinished()
{
    this->onsettingschanged();
}

void MainWindow::on_upquota_editingFinished()
{
    this->onsettingschanged();
}

void MainWindow::on_totalquota_editingFinished()
{
    this->onsettingschanged();
}

void MainWindow::on_downquotaunit_currentIndexChanged(int index)
{
    if (this->has_initialised)
        ui->downquota->setValue(this->config.download_quota / this->get_unit(ui->downquotaunit));
}

void MainWindow::on_upquotaunit_currentIndexChanged(int index)
{
    if (this->has_initialised)
        ui->upquota->setValue(this->config.upload_quota / this->get_unit(ui->upquotaunit));
}

void MainWindow::on_totalquotaunit_currentIndexChanged(int index)
{
    if (this->has_initialised)
        ui->totalquota->setValue(this->config.total_quota / this->get_unit(ui->totalquotaunit));
}

void MainWindow::on_monthrotate_editingFinished()
{
    this->onsettingschanged();
}

void MainWindow::on_base2radio_toggled(bool checked)
{
    this->onsettingschanged();
}

void MainWindow::on_base10radio_toggled(bool checked)
{
    this->onsettingschanged();
}

void MainWindow::on_tabWidget_currentChanged(int index)
{
    if (index == 0) {
        this->refresh();
    }
    if (index == 1) {
        this->draw_chart();
    }
}

void MainWindow::on_actionOverview_triggered()
{
    ui->tabWidget->setCurrentIndex(0);
}

void MainWindow::on_actionCharts_triggered()
{
    ui->tabWidget->setCurrentIndex(1);
}

void MainWindow::on_actionPreferences_triggered()
{
    ui->tabWidget->setCurrentIndex(2);
}

void MainWindow::on_action_About_triggered()
{
    AboutDialog(this);
}

void MainWindow::on_actionReport_Bug_triggered()
{
    system("xdg-open https://bugs.launchpad.net/bitbudget/+filebug");
}

void MainWindow::on_actionOnline_Support_triggered()
{
    system("xdg-open https://answers.launchpad.net/bitbudget/+addquestion");
}

MainWindowAdaptor::MainWindowAdaptor (MainWindow *parent) : QDBusAbstractAdaptor(parent) {

}

void MainWindowAdaptor::process_argument (QString arg) {
    QStringList args;
    args.append(arg.prepend("--"));
    MainWindow *that = qobject_cast<MainWindow *>(this->parent());
    that->process_arguments(args);
}


void MainWindow::update_launcher (BitBudget::DataBase::overview_data o) {
    QList<QAction*> actions;
    qreal total1 = o.monthly_rx;
    if (config.download_quota > 0) {
        total1 /= config.download_quota;
    } else {
        total1 = 0.0;
    }
    qreal total2 = o.monthly_tx;
    if (config.upload_quota > 0) {
        total2 /= config.upload_quota;
    } else {
        total2 = 0.0;
    }
    if (total2 > total1) {
        total1 = total2;
    }
    total2 = o.monthly_rx + o.monthly_tx;
    if (config.total_quota > 0) {
        total2 /= config.total_quota;
    } else {
        total2 = 0.0;
    }
    if (total2 > total1) {
        total1 = total2;
    }
    if (total1 > 1.0) {
        total1 = 1.0;
    }
    if (total1 < 0.0) {
        total1 = 0.0;
    }
    QDBusMessage signal = QDBusMessage::createSignal("/", "com.canonical.Unity.LauncherEntry", "Update");
    signal << "application://bitbudget.desktop";

    //Menu
    quicklist->clear();
    actions.append(new QAction(ui->l_h_t->text().prepend("Hourly: "), quicklist));
    QObject::connect(actions[0], SIGNAL(triggered()), this, SLOT(quicklist_hourly_activate()));
    actions.append(new QAction(ui->l_d_t->text().prepend("Daily: "), quicklist));
    QObject::connect(actions[1], SIGNAL(triggered()), this, SLOT(quicklist_daily_activate()));
    actions.append(new QAction(ui->l_m_t->text().prepend("Monthly: "), quicklist));
    QObject::connect(actions[2], SIGNAL(triggered()), this, SLOT(quicklist_monthly_activate()));
    quicklist->addActions(actions);

    //progress bar
    signal_properties["progress"] = double(total1);

    signal << signal_properties;
    QDBusConnection::sessionBus().send(signal);

}
